# coding: utf-8

from .base import check_files


def check(filename, skip_keys=None):
    """
    Вернуть список с непереведенными ключами.

    @type filename: str
    @type skip_keys: list
    @rtype: list
    """
    return list(check_files([filename], skip_keys))



# -*- coding: utf-8 -*-

import os
import sys
from collections import namedtuple
from optparse import make_option
from django.conf import settings
from django.core.management.base import BaseCommand

from po_file_checker.base import check_files

Domain = namedtuple('domain', ['name', 'must_exist'])
DOMAINS = (
    Domain('django', must_exist=True),
    Domain('djangojs', must_exist=False),
)


ERROR_MISSING_FILES = 1
ERROR_FILE_DOES_NOT_EXIST = 2


class Command(BaseCommand):
    help = "Check all po-files in project"
    requires_model_validation = False

    option_list = BaseCommand.option_list + (
        make_option(
            '--skip', '-s',
            action='append',
            dest='skip',
            help='Don\'t check the key (may be applied multiple times)'),
    )

    def handle(self, *args, **options):
        verbosity = int(options['verbosity'])

        missing = tuple(check_files(
            self.generate_filenames(verbosity),
            skip_keys=options['skip'] or tuple()
        ))

        if missing:
            print "\n".join(missing)
            sys.exit(ERROR_MISSING_FILES)

    def generate_filenames(self, verbosity):
        """
        Вернуть список имен po-файлов.

        @type verbosity: int
        @rtype: list
        """
        filenames = []

        for domain in DOMAINS:
            localedir = os.path.abspath('locale')
            languages = [lang[0] for lang in settings.LANGUAGES]
            for locale in languages:
                basedir = os.path.join(localedir, locale, 'LC_MESSAGES')
                pofile = os.path.join(basedir, '%s.po' % domain.name)

                if verbosity > 0:
                    print 'processing file "{0}"'.format(pofile)

                if domain.must_exist and not os.path.exists(pofile):
                    sys.stderr.writelines(
                        'pofile is obligatory and does not exist: "{0}"'.format(
                            pofile
                        )
                    )
                    sys.exit(ERROR_FILE_DOES_NOT_EXIST)

                filenames.append(pofile)

        return filenames



# coding: utf-8

import polib


def entries_with_missing_translations(po):
    """
    Генератор непереведенных записей в po-файле.

    @type po: POFile
    @rtype: POEntry
    """
    for entry in po:
        if not entry.translated():
            if not entry.occurrences:
                continue
            if not entry.msgstr and entry.msgstr_plural:
                # У некоторых записей есть msgstr_plural, но нет msgstr.
                # Их считаем переведенными, если они не obsolete и не fuzzy!
                if not entry.obsolete and 'fuzzy' not in entry.flags:
                    continue
            yield entry


def missing_keys(po):
    """
    Генератор непереведенных ключей.

    @type po: POFile
    @rtype: str
    """
    for entry in entries_with_missing_translations(po):
        if not entry.msgstr:
            yield entry.msgid
        else:
            yield entry.msgid_plural


def check_files(filenames, skip_keys=None):
    """
    Генератор отсутствующих ключей из po-файлов.

    @type filenames: iterable
    @param skip_keys: Пропускать заданные ключи.
    @type skip_keys: list
    @rtype: str
    """
    if skip_keys is None:
        skip_keys = tuple()
    for filename in filenames:
        for key in missing_keys(polib.pofile(filename)):
            if key in skip_keys:
                continue
            yield key



# coding: utf-8

from setuptools import setup, find_packages

PACKAGE_ROOT = 'src'
PACKAGE_NAME = 'po_file_checker'

setup(
    name=PACKAGE_NAME,
    version='0.2',
    description='Check that po-files have no untranslated keys',
    author_email='chapson@yandex-team.ru',
    keywords="po-file, build-tools, sanity check",
    package_dir={'': PACKAGE_ROOT},
    packages=find_packages(PACKAGE_ROOT, exclude=['tests']),
    install_requires=[
        'polib',
        'mock',
        'nose',
    ]
)



#!/usr/bin/env python
# coding: utf-8

import sys
from po_file_checker.base import check_files

if __name__ == '__main__':
    missing = tuple(check_files(sys.argv[1:]))
    if missing:
        sys.stderr.write(
            "\n".join(missing)
        )
        sys.exit(1)

    sys.exit(0)
